#include "HerbivoreANN.h"
#include "util.h"


void HerbivoreANN::initANN(bool randomize){
	vector<int> layout = {2,4,5,2,1};
	neural = new ANN(layout,randomize);
}

//Napraviti kromosom
HerbivoreANN::HerbivoreANN(World* box2DWorld, const CreatureConfiguration& configuration, World* parent) :Herbivore(box2DWorld, configuration, parent)
{
	b2BodyDef bodydef;
	bodydef.position.Set(configuration.position.x, configuration.position.y);
	bodydef.type = b2_dynamicBody;

	initANN(true);

	m_body = m_box2DWorld->CreateBody(&bodydef);
	m_body->SetUserData(this);

	m_createShape(configuration.size);
}

HerbivoreANN::HerbivoreANN(const b2Vec2& pos, float size, World *box2DWorld, World *parent) : Herbivore(pos, size, box2DWorld, parent)
{
	b2BodyDef bodydef;
	bodydef.position.Set(pos.x, pos.y);
	bodydef.type = b2_dynamicBody;

	initANN(false);
	//making of weigths is handled in creature part because of reproduce is stationed there
	m_body = m_box2DWorld->CreateBody(&bodydef);
	m_body->SetUserData(this);

	m_createShape(size);
}

Creature::Type HerbivoreANN::type()
{
	return Creature::Type::HERBIVOREANN;
}



//making decision will replace explore,because the creature has the ability to find closest target
void HerbivoreANN::Step()
{
	Creature::Step();

	bool busy = false;


	Creature* food = getClosestVisible(PLANT);
	Creature* mate = getClosestVisible(HERBIVOREANN);
	//Creature* danger = getClosestVisible(PREDATORANN);

	vector<float> inputs = { 1e10, 1e10 };

	if (food)
		inputs[0] = Util::distance(this->position(), food->position());
	if (mate)
		inputs[1] = Util::distance(this->position(), mate->position());
	//if (danger)
		//inputs[3] = Util::distance(this->position(), danger->position());
	float decision = neural->getOutput(inputs);
	bool eat = decision < 0.5;

	if (eat && food || !canReproduce() && food){
		go_to_Creature(food);
	}
	else if (mate)
	{
		go_to_Creature(mate);
	}


	if (!m_touchingCreatures.isEmpty())
	{
		foreach(Creature *creature, m_touchingCreatures)
		{
			switch (creature->type())
			{
			case HERBIVOREANN:
				// check for reproduction ability
				if (canReproduce() && creature->canReproduce() && !eat)
				{
					busy = true;
					reproduce(creature);
				}
				break;

			case PLANT:
				// eat if lack energy
				if (eat || !canReproduce())//just if eat
				{
					busy = true;
					creature->paralyze();
					creature->shrink();
					m_energy += creature->takeEnergy(this);
				}
				break;
			}
		}
	}

}

void HerbivoreANN::go_to_Creature(Creature* target) {
	b2Vec2 toTarget = target->position() - m_body->GetPosition();
	float desiredAngle = atan2(toTarget.y, toTarget.x);
	turnTo(desiredAngle);
	go(50);
}

//check where explore is called and reroute it to step
void HerbivoreANN::explore()
{
	Creature::explore();

	if (energyPercentage() > 1.2) return;

	//Plant
	HerbivoreANN *herbivore = (HerbivoreANN *)getClosestVisible(HERBIVORE);
	if (herbivore)
	{
		b2Vec2 toTarget = herbivore->position() - m_body->GetPosition();
		float desiredAngle = atan2(toTarget.y, toTarget.x);
		turnTo(desiredAngle);
		go(50);
	}
}


void HerbivoreANN::m_createShape(double size)
{
	const float DEG2RAD = 3.14159f / 180.0f;
	QVector<b2Vec2> vertices;
	for (int i = 0; i < 360; i += 120)
	{
		float degInRad = i*DEG2RAD;
		vertices.append(b2Vec2(cos(degInRad)*size, sin(degInRad)*size));
	}
	b2PolygonShape shape;
	shape.Set(vertices.data(), vertices.length());

	b2FixtureDef fixturedef;
	fixturedef.shape = &shape;
	fixturedef.density = 0.5;
	m_body->CreateFixture(&fixturedef);
	m_body->SetAngularDamping(0.5);
	m_body->SetLinearDamping(0.5);

	createSensor(10 * size);
}

