#include "PredatorANN.h"
#include "util.h"
#include <qdebug.h>


void PredatorANN::initANN(bool randomize){
	vector<int> layout = { 2, 5, 3, 1 };
	neural = new ANN(layout, randomize);
}

//Napraviti kromosom
PredatorANN::PredatorANN(World* box2DWorld, const CreatureConfiguration& configuration, World* parent) : Predator(box2DWorld, configuration, parent)
{
	b2BodyDef bodydef;
	bodydef.position.Set(configuration.position.x, configuration.position.y);
	bodydef.type = b2_dynamicBody;

	initANN(true);

	m_body = m_box2DWorld->CreateBody(&bodydef);
	m_body->SetUserData(this);

	m_createShape(configuration.size);
}

PredatorANN::PredatorANN(const b2Vec2& pos, float size, World *box2DWorld, World *parent) : Predator(pos, size, box2DWorld, parent)
{
	b2BodyDef bodydef;
	bodydef.position.Set(pos.x, pos.y);
	bodydef.type = b2_dynamicBody;

	initANN(false);
	//making of weigths is handled in creature part because of reproduce is stationed there
	m_body = m_box2DWorld->CreateBody(&bodydef);
	m_body->SetUserData(this);

	m_createShape(size);
}

Creature::Type PredatorANN::type()
{
	return Creature::Type::PREDATORANN;
}



//making decision will replace explore,because the creature has the ability to find closest target
void PredatorANN::Step()
{
	Creature::Step();

	bool busy = false;


	//// if energy low, check if Herbivores nearby
	//if (energyPercentage() < 0.50)
	//{
	//	Herbivore *herbivore = (Herbivore *)getClosestVisible(HERBIVORE);
	//	if (herbivore)
	//	{
	//		busy = true;
	//		// go eat
	//		b2Vec2 fromTarget = herbivore->position() - m_body->GetPosition();
	//		float desiredAngle = atan2(fromTarget.y, fromTarget.x);
	//		turnTo(desiredAngle);
	//		go(150);
	//	}
	//}

	//// if all is well, look for predators to reproduce with
	//if (canReproduce())
	//{
	//	Predator *predator = (Predator *)getClosestVisible(PREDATORANN);
	//	if (predator)
	//	{
	//		busy = true;
	//		// go eat
	//		b2Vec2 fromTarget = predator->position() - m_body->GetPosition();
	//		float desiredAngle = atan2(fromTarget.y, fromTarget.x);
	//		turnTo(desiredAngle);
	//		go(50);
	//	}
	//}

	// touchy feely

	//replace with ANN decision eat or reproduce
	//ANN inputs are current size,energy,distance to closest food and reproductive partner
	//also turn of explore option,ANN makes decision
	
	
	//ANN pass inputs get decision,output > 0.5
	
		Creature* food = getClosestVisible(HERBIVOREANN);
		Creature* mate = getClosestVisible(PREDATORANN);
		vector<float> inputs(2);
		inputs[0] = 1e10;
		inputs[1] = 1e10;
		if (food)
		inputs[0] = Util::distance(this->position(), food->position());
		if (mate)
		inputs[1] = Util::distance(this->position(), mate->position());
		float out = neural->getOutput(inputs);
		bool eat =  out > 0.5;
		//qDebug() << out << endl;
		if (eat && food || !canReproduce() && food){
			go_to_Creature(food);
		}
		else if (mate)
		{
			go_to_Creature(mate);
		}
		

	
	

	if (!m_touchingCreatures.isEmpty())
	{
		foreach(Creature *creature, m_touchingCreatures)
		{
			switch (creature->type())
			{
			case PREDATORANN:
				// check for reproduction ability
				if (canReproduce() && creature->canReproduce()&& !eat)
				{
					busy = true;
					reproduce(creature);
				}
				break;

			case HERBIVOREANN:
				// eat if lack energy
				if (eat || !canReproduce())//just if eat
				{
					busy = true;
					creature->paralyze();
					creature->shrink();
					m_energy += creature->takeEnergy(this);
				}
				break;

			case PLANT:
			default:
				break;
			}
		}
	}

}

void PredatorANN::go_to_Creature(Creature* target) {
	b2Vec2 toTarget = target->position() - m_body->GetPosition();
	float desiredAngle = atan2(toTarget.y, toTarget.x);
	turnTo(desiredAngle);
	go(50);
}

//check where explore is called and reroute it to step
void PredatorANN::explore()
{
	Creature::explore();

	if (energyPercentage() > 1.2) return;

	Herbivore *herbivore = (Herbivore *)getClosestVisible(HERBIVORE);
	if (herbivore)
	{
		b2Vec2 toTarget = herbivore->position() - m_body->GetPosition();
		float desiredAngle = atan2(toTarget.y, toTarget.x);
		turnTo(desiredAngle);
		go(50);
	}
}


// b2Vec2 Predator::forceDirection()
// {
//     b2Vec2 front, back;
//     b2PolygonShape* shape = (b2PolygonShape*)body()->GetFixtureList()->GetShape();
//     int vertexCount = shape->GetVertexCount();
//     
//     front = shape->GetVertex(0);
//     back = shape->GetVertex(2) + 0.5 * ( shape->GetVertex(2) - shape->GetVertex(1) );
//     
//     b2Vec2 vec = back - front;
//     vec = 1./sqrt(vec.x*vec.x + vec.y*vec.y) * vec;
//     
//     // rotate vector to fit the body orientation
//     vec.x = vec.x * cos( body()->GetAngle() ) - vec.y * sin( body()->GetAngle() );
//     vec.y = vec.x * sin( body()->GetAngle() ) + vec.y * cos( body()->GetAngle() );
//     
//     return vec;
// }


void PredatorANN::m_createShape(double size)
{
	const float DEG2RAD = 3.14159f / 180.0f;
	QVector<b2Vec2> vertices;
	for (int i = 0; i < 360; i += 120)
	{
		float degInRad = i*DEG2RAD;
		vertices.append(b2Vec2(cos(degInRad)*size, sin(degInRad)*size));
	}
	b2PolygonShape shape;
	shape.Set(vertices.data(), vertices.length());

	b2FixtureDef fixturedef;
	fixturedef.shape = &shape;
	fixturedef.density = 0.5;
	m_body->CreateFixture(&fixturedef);
	m_body->SetAngularDamping(0.5);
	m_body->SetLinearDamping(0.5);

	createSensor(10 * size);
}

