#ifndef CREATURE_H
#define CREATURE_H

#include <QObject>
#include <Box2D/Box2D.h>

#include "creatureconfiguration.h"

class World;
class Creature : public QObject
{
    Q_OBJECT
    
public:
    Creature(World *world, const CreatureConfiguration &configuration, World *parent = 0);
	Creature(const b2Vec2& pos, float size, World *world, World *parent = 0);
    ~Creature();
    enum Type
    {
        DEFAULT,
        PLANT,
        HERBIVORE,
        PREDATOR,
		PREDATORANN,
		HERBIVOREANN,
        CUSTOM
    };
    
    virtual Type type();
    virtual void Step();
    
    virtual bool isDead();
    virtual bool canReproduce();
    float energyPercentage();
    float energy();
    
    float size();
    float mass();
    
    virtual void createSensor( float size );
    
    virtual QString toString();
    
    b2Body *body();
    
    virtual b2Vec2 forceDirection();
    void paralyze( float time = 2 );
	Creature* searchQuadrant(int x, int y,Type type);
	Creature* getClosestVisible(Creature* a, Creature* b);
    bool isParalyzed();
    void go( float force );
    void turn( float torque );
    void turnTo( float angle ); // starts turning; continually update to turn completely
    
    virtual void explore();
    
    void grow( float growthRate = 0 );
    void shrink( float shrinkRate = 0 );
    
    virtual float takeEnergy( Creature *taker );
    
    b2Vec2 position();
    
    
    /**
     * @mutationRate should be between 0 and 1, where 0 is for no mutation
     */
    Creature *reproduce( Creature* other );
    
    void addToNearBy(Creature* creature);
    void addToTouching(Creature* creature);
    void removeFromNearBy(Creature* creature);
    void removeFromTouching(Creature* creature);
    
    Creature *getClosestVisible( Type type );
    
    b2World *m_box2DWorld;
    b2Body *m_body;
	World *world;
    
    QList<Creature *> m_nearByCreatures;
    QList<Creature *> m_touchingCreatures;
    
    float m_paralyzationTimer;
    float m_initialEnergy;
    float m_energy;
    
    float m_growthRate;
    float m_age;
    float m_maxAge;
    float m_mutationRate;
    
    float m_reproductionTime;
    float m_reproductionTimeThreshold;
    
    float m_initialSize;
};

#endif
 
