#include "world.h"

#include <Box2D/Box2D.h>
#include <QFile>
#include <QSettings>
#include <stdlib.h>
#include "plant.h"
#include "HerbivoreANN.h"
#include <QtDebug>

bool ANN_test = true;

World::World(QObject *parent)
    : QObject(parent)
{
    m_box2DWorld = new b2World(b2Vec2_zero);
	
	grid.resize(wh);
	for (int i = 0; i < wh; i++)
	{
		grid[i].resize(wh);
	}
}

World::~World()
{
    delete m_box2DWorld;
}

void World::save(QString filename)
{
    QFile file( filename );
    file.open( QIODevice::WriteOnly );
    
    file.write(QString("%1 %2\n").arg(m_size.x).arg(m_size.y).toUtf8());
    
    foreach( Creature *creature, m_population )
    {
	file.write( QString("%1\n").arg(creature->toString()).toUtf8() );
    }
    
    file.flush();
    file.close();
}


Plant* World::addPlant(const CreatureConfiguration& configuration)
{
//	qDebug() << (configuration.position.x / m_size.x)*wh << " " << configuration.position.x << " " << m_size.x << endl;
    Plant *plant = new Plant(this, configuration, this);
	int x = (int)((configuration.position.x / m_size.x) * wh), y=(int)((configuration.position.y / m_size.y) * wh);
	grid[x < wh ? x : --x][y < wh ? y : --y].push_back(plant);
    m_population.append(plant);
    return plant;
}

Plant* World::addPlant(const b2Vec2& pos, float size)
{
//	qDebug() << (pos.x / m_size.x)*wh << " " << pos.x << " " << m_size.x << endl;
    Plant *plant = new Plant(pos, size, this, this);
	int x = (int)((pos.x / m_size.x) * wh), y = (int)((pos.y / m_size.y) * wh);
	grid[x < wh ? x : --x][y < wh ? y : --y].push_back(plant);
    m_population.append(plant);
    return plant;
}


Herbivore* World::addHerbivore(const CreatureConfiguration& configuration)
{
//	qDebug() << (configuration.position.x / m_size.x)*wh << " " << configuration.position.x << " " << m_size.x << endl;
	Herbivore *herbivore;
	if (ANN_test){
		herbivore = (Herbivore *) new HerbivoreANN(this, configuration, this);
	}
	else{
		herbivore = new Herbivore(this, configuration, this);
	}
	int x = (int)((configuration.position.x / m_size.x) * wh), y=(int)((configuration.position.y / m_size.y) * wh);
	grid[x < wh ? x : --x][y < wh ? y : --y].push_back(herbivore);
    m_population.append(herbivore);
    return herbivore;
}

Herbivore* World::addHerbivore(const b2Vec2& pos, float size)
{
//	qDebug() << (pos.x / m_size.x)*wh << " " << pos.x << " " << m_size.x << endl;
	Herbivore *herbivore;
	if (ANN_test){
		herbivore = (Herbivore *) new HerbivoreANN(pos, size, this, this);
	}
	else{
		herbivore = new Herbivore(pos, size, this, this);
	}
	int x = (int)((pos.x / m_size.x) * wh), y = (int)((pos.y / m_size.y) * wh);
	grid[x < wh ? x : --x][y < wh ? y : --y].push_back(herbivore);
    m_population.append(herbivore);
    return herbivore;
}


Predator* World::addPredator(const CreatureConfiguration& configuration)
{
	Predator* predator;
	if (ANN_test) 
	{
		PredatorANN* temp = new PredatorANN(this,configuration,this);
		predator = temp;
	}
	else
	{
		predator =  new Predator(this, configuration, this);
	}
	int x = (int)((configuration.position.x / m_size.x) * wh), y = (int)((configuration.position.y / m_size.y) * wh);

	grid[x < wh ? x : --x][y < wh ? y : --y].push_back(predator);
    m_population.append(predator);
    return predator;
}

Predator* World::addPredator(const b2Vec2& pos, float size)
{
	Predator *predator = new Predator(pos, size, this, this);
	if (ANN_test){
		PredatorANN* temp = new PredatorANN(pos, size, this, this);
	}
	int x = (int)pos.x / (int)m_size.x * wh, y = (int)((pos.y / m_size.y) * wh);
	grid[x < wh ? x : --x][y < wh ? y : --y].push_back(predator);
    m_population.append(predator);
    return predator;
}

void World::addCreature(Creature* creature)
{
	int x = (int)((creature->position().x / m_size.x) * wh), y = (int)((creature->position().y / m_size.y) * wh);
	grid[x < wh ? x : --x][y < wh ? y : --y].push_back(creature);
    m_population.append(creature);
}

void World::clearInhabitants()
{
    foreach( Creature *creature, m_population )
    {
	m_box2DWorld->DestroyBody( creature->body() );
    }
    m_population.clear();
}


void World::createBorder(const b2Vec2& size)
{
    // destroy old border
    for( int i = 0; i < m_border.size(); i++ )
    {
	b2Body *b = m_border[i];
	m_box2DWorld->DestroyBody(b);
    }
    m_border.clear();
    
    m_size = size;
    
    // create new border
    b2BodyDef bodydef;
    bodydef.position.Set(size.x/2, -5);
    bodydef.type = b2_staticBody;
    bodydef.angle = 0;
    
    b2PolygonShape boxShape;
    boxShape.SetAsBox( size.x/2 + 10, 5 );
    
    b2Body *bodyTop = m_box2DWorld->CreateBody(&bodydef);
    

    b2FixtureDef fixturedef;
    fixturedef.shape = &boxShape;
    fixturedef.density=1.0;
    bodyTop->CreateFixture(&fixturedef);
    
    m_border.append( bodyTop );
    
    bodydef.position.Set( size.x/2, size.y + 5 );
    b2Body *bodyBottom = m_box2DWorld->CreateBody(&bodydef);
    bodyBottom->CreateFixture(&fixturedef);
    m_border.append(bodyBottom);
    
    bodydef.position.Set( -5, size.y/2 );
    boxShape.SetAsBox( 5, size.y/2 );
    b2Body *bodyLeft = m_box2DWorld->CreateBody(&bodydef);
    bodyLeft->CreateFixture(&fixturedef);
    m_border.append(bodyLeft);
    
    bodydef.position.Set( size.x + 5, size.y/2 );
    boxShape.SetAsBox( 5, size.y/2 );
    b2Body *bodyRight = m_box2DWorld->CreateBody(&bodydef);
    bodyRight->CreateFixture(&fixturedef);
    m_border.append(bodyRight);
}




void World::Step(float timeStep, int velocityIterations, int positionIterations)
{
	vector<vector<vector<Creature *>>> tmp_grid(wh);
	
	for (int i = 0; i < wh; i++)
	{
		tmp_grid[i].resize(wh);
	}

	foreach( Creature* c, m_population )
    {
	c->Step();
	
	// remove the dead, show must go on
	if( c->isDead() )
	{
	    m_box2DWorld->DestroyBody(c->body());
	    m_population.removeAll(c);
	}
	else
	{
		tmp_grid[(c->position().x / m_size.x)*wh][(c->position().y / m_size.y)*wh].push_back(c);
	}
    }

	grid.clear();
	grid = tmp_grid;

    m_box2DWorld->Step(timeStep, velocityIterations, positionIterations);
}


void World::SetContactListener(b2ContactListener* contactListener)
{
    m_box2DWorld->SetContactListener(contactListener);
}



const QList< Creature* >& World::population()
{
    return m_population;
}

const QList< b2Body* >& World::border()
{
    return m_border;
}

b2Vec2 World::size()
{
    return m_size;
}
